﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using GalaSoft.MvvmLight.Messaging;

namespace Shell
{
    [Export(typeof(IDeploymentService))]
    public class DeploymentCatalogService : IDeploymentService
    {
        private static AggregateCatalog aggregateCatalog;

        readonly Dictionary<string, DeploymentCatalog> _catalogs;

        public DeploymentCatalogService()
        {
            _catalogs = new Dictionary<string, DeploymentCatalog>();
        }

        public static void Initialize()
        {
            aggregateCatalog = new AggregateCatalog();
            aggregateCatalog.Catalogs.Add(new DeploymentCatalog());
            CompositionHost.Initialize(aggregateCatalog);
        }

        public void AddXap(string uri, Action<AsyncCompletedEventArgs> completedAction = null)
        {
            DeploymentCatalog catalog;
            if (!_catalogs.TryGetValue(uri, out catalog))
            {
                catalog = new DeploymentCatalog(uri);
                if (completedAction != null)
                    catalog.DownloadCompleted += (s, e) => completedAction(e);
                else
                    catalog.DownloadCompleted += catalog_DownloadCompleted;

                catalog.DownloadAsync();
                _catalogs[uri] = catalog;
            }
            aggregateCatalog.Catalogs.Add(catalog);
        }

        static void catalog_DownloadCompleted(object sender, AsyncCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                Messenger.Default.Send<NotificationMessage<Exception>>(new NotificationMessage<Exception>(e.Error,"Exception"));
                //throw new Exception(e.Error.Message, e.Error);
            }
        }

        public void RemoveXap(string uri)
        {
            DeploymentCatalog catalog;
            if (_catalogs.TryGetValue(uri, out catalog))
            {
                aggregateCatalog.Catalogs.Remove(catalog);
            }
        }
    }

}
