﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;

namespace Shell
{
    public class EnumCollection<T> : List<EnumContainer> where T : struct
    {
        public EnumCollection()
        {
            var type = typeof(T);
            if (!type.IsEnum)
                throw new ArgumentException("This class only supports Enum types");
            var fields = typeof(T).GetFields(BindingFlags.Static | BindingFlags.Public);
            foreach (var field in fields)
            {
                var container = new EnumContainer
                                    {
                                            EnumOriginalValue = field.GetValue(null),
                                            EnumValue = (int) field.GetValue(null),
                                            EnumDescription = field.Name
                                    };
                var atts = field.GetCustomAttributes(false);
                foreach (var att in atts)
                    if (att is DescriptionAttribute)
                    { container.EnumDescription = ((DescriptionAttribute)att).Description; break; }
                Add(container);
            }
        }
    }
}
