﻿using System;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using Contract;
using GalaSoft.MvvmLight.Messaging;

namespace Shell
{
    public partial class MainPage : UserControl, IPartImportsSatisfiedNotification
    {
        public MainPage()
        {
            InitializeComponent();
            CompositionInitializer.SatisfyImports(this);
            DataContext = ViewModel;
            Messenger.Default.Register<NotificationMessage>(this, delegate { displayWidgets(); });
            Messenger.Default.Register<NotificationMessage<Exception>>(this, (n) =>
                                                                                 {
                                                                                     if (n.Notification == "Exception") showException(n.Content);
                                                                                 });
        }


        [Import]
        public MainPageViewModel ViewModel { get; set; }

        public void OnImportsSatisfied()
        {
            displayWidgets();
        }

        private void displayWidgets()
        {
            MainContainer.Children.Clear();
            SecondaryContainer.Children.Clear();
            Dispatcher.BeginInvoke(() =>
                                       {
                                           try
                                           {
                                               foreach (var widget in ViewModel.Widgets)
                                               {
                                                   if (widget.Metadata.Area == WidgetArea.TopArea) MainContainer.Children.Add(widget.Value);
                                                   else SecondaryContainer.Children.Add(widget.Value);
                                               }
                                           }
                                           catch (Exception e)
                                           {
                                              showException(e);
                                           }
                                       });
        }

        /* exceptions */

        private void showException(Exception exception)
        { Dispatcher.BeginInvoke(() => MessageBox.Show("Exception:" + Environment.NewLine + exception.Message)); }
    }
}
