﻿using System;
using System.Net;
using System.Windows.Controls;
using System.ComponentModel.Composition;
using System.ComponentModel;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using System.Xml.Linq;
using Contract;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Messaging;

namespace Shell
{
    [Export]
    public class MainPageViewModel : ViewModelBase, IPartImportsSatisfiedNotification
    {


        [ImportMany(AllowRecomposition = true)]
        public Lazy<UserControl, IWidgetMetadata>[] Widgets { get; set; }

        public IEnumerable<WidgetInfo> WidgetsInfo { get; set; }

        public void OnImportsSatisfied()
        {
            var l = new List<WidgetInfo>();
            foreach (var widget in Widgets)
            {
                var w = new WidgetInfo
                            {
                                    Area = widget.Metadata.Area,
                                    WidgetName = widget.Metadata.Name,
                                    IsSelected = false,
                                    IsActive = true,
                                    Version = widget.Metadata.Version
                            };

                l.Add(w);
            }
            WidgetsInfo = l.OrderBy(x => x.WidgetName).ToList();
            RaisePropertyChanged("WidgetsInfo");
            Messenger.Default.Send(new NotificationMessage("UpdateDisplay"));
        }

       

        /* user role */

        private UserRole currentUserRole = UserRole.Unknown;
        public UserRole CurrentUserRole
        {
            get { return currentUserRole; }
            set
            {
                if (currentUserRole == value) return;
                currentUserRole = value;
                RaisePropertyChanged("CurrentUserRole");
                LoadModules(currentUserRole);
            }
        }

        /* Load extra modules */

        public ICommand LoadExtraModules { get; private set; }

        public MainPageViewModel()
        {
            LoadExtraModules = new RelayCommand(()=>LoadModules(CurrentUserRole));
        }

        /* catalog */

        [Import(typeof(IDeploymentService))]
        public IDeploymentService Service { get; set; }

        private void LoadModules(UserRole userRole)
        {
            var wc = new WebClient();
            wc.OpenReadCompleted += (s, e) =>
            {
                if (e.Error!=null)
                {
                    Messenger.Default.Send(new NotificationMessage<Exception>(e.Error,"Exception"));
                    return;
                }
                var streamInfo = e.Result;

                var xElement = XElement.Load(streamInfo);

                var modulesList = from m in xElement.Elements("ModuleInfo")
                                  select m;
                if (!modulesList.Any()) return;
                foreach (var module in modulesList)
                {
                    var roleAttribute = module.Attribute("UserRole");
                    if (roleAttribute == null) continue; // no role, no module = security.
                    if (string.Compare(roleAttribute.Value,userRole.ToString(), StringComparison.InvariantCultureIgnoreCase)!=0)
                        continue; // user role mismatch, no module.

                    var moduleAttribute = module.Attribute("XapFilename");
                    if (moduleAttribute != null)
                    {
                        var moduleName = moduleAttribute.Value;

                        Service.AddXap(moduleName,null);
                    }
                }
            };
            wc.OpenReadAsync(new Uri("Catalog.xml", UriKind.Relative));
        }

    }
}
